from datetime import datetime

from pydantic import BaseModel


class PartMetadataSchema(BaseModel):
    PartNumber: int
    ETag: str


class FileMetaData(BaseModel):
    file_size: int
    content_type: str
    last_modified: datetime
