from setuptools import find_packages, setup

setup(
    name="s3-upload-manager",
    version="0.1.0",
    packages=find_packages(exclude=["tests*"]),
    install_requires=["pydantic-settings>=2.0.0", "botocore>=1.42.36"],
    python_requires=">=3.9",
)
